;******************************************************************************
; FLICKER                                                                     *
;                                                                             *
; Make up to five different lamps flicker pseudorandomly.                     *
;                                                                             *
;******************************************************************************
;                                                                             *
; Pin assignments:                                                            *
;   GP0 - first light.                                                        *
;   GP1 - second light.                                                       *
;   GP2 - third light.                                                        *
;   GP4 - fourth light.                                                       *
;   GP5 - fifth light.                                                        *
;                                                                             *
;******************************************************************************
;                                                                             *
; Copyright (C) 2012 Sergio García-Cuevas González.                           *
;                                                                             *
; This program is free software; you can redistribute it and/or modify        *
; it under the terms of the GNU General Public License as published by        *
; the Free Software Foundation; either version 3 of the License, or           *
; (at your option) any later version.                                         *
;                                                                             *
; This program is distributed in the hope that it will be useful,             *
; but WITHOUT ANY WARRANTY; without even the implied warranty of              *
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
; GNU General Public License for more details.                                *
;                                                                             *
; You should have received a copy of the GNU General Public License           *
; along with this program.  If not, see <http://www.gnu.org/licenses/>.       *
;                                                                             *
;******************************************************************************

        list      p=12f629
        #include <p12f629.inc>

;***** CONFIGURATION
; _CPD_OFF:            Off course you can copy data.
; _CP_OFF:             Off course you can copy code.
; _BODEN_ON:           Brown-out detection is enabled.
; _MCLRE_OFF:          Reset button disabled.
; _PWRTE_ON:           Wait 72 ms at power-up for supply-voltage stabilisation.
; _WDT_OFF:            Watchdog timer is off.
; _INTRC_OSC_NOCLKOUT: Use internal oscillator.
       __CONFIG  _CPD_OFF & _CP_OFF & _BODEN_ON & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT

; 32-bit LFSR.
lfsr1   EQU     0x20
lfsr2   EQU     0x21
lfsr3   EQU     0x22
lfsr4   EQU     0x23

; 16-bit counter.
cntr1   EQU     0x24
cntr2   EQU     0x25

; Starting value of the LFSR.
start1  EQU     b'00000000'
start2  EQU     b'00000000'
start3  EQU     b'00000000'
start4  EQU     b'00000001'

; Taps for a maximum-length, 32-bit LFSR.
tap1    EQU     b'11111111'
tap2    EQU     b'11111111'
tap3    EQU     b'11111111'
tap4    EQU     b'11101110'

; Starting values of the outer counter.
iter2    EQU     0x10

reset
        ORG     0x000
        CLRF    cntr1
        CLRF    cntr2
        MOVLW   start1
        MOVWF   lfsr1
        MOVLW   start2
        MOVWF   lfsr2
        MOVLW   start3
        MOVWF   lfsr3
        MOVLW   start4
        MOVWF   lfsr4
        ; Oscillator calibration.
        CALL    0x3ff       ; Load oscillator calibration.
        BSF     STATUS,RP0  ; Select bank 1 for OSCCAL and TRISIO.
        MOVWF   OSCCAL      ; Apply the calibration.
        ; Inputs.
        CLRF    TRISIO      ; No inputs.
        ; Comparator.
        BCF     STATUS,RP0  ; Go to bank 0 for CMCON.
        BANKSEL CMCON
        MOVLW   b'00000111' ; CMCON configuration.
                ; -----111    CM2:CM0: comparator off.
        MOVWF   CMCON

; Main loop.
main_loop
        CALL    lfsr        ; Generate a new pseudorandom number.
        MOVF    lfsr1,W     ; We use the first word of the LFSR as the new
        MOVWF   GPIO        ; value of GPIO, but any other word of the LFSR
                            ; would have sufficed.

; Busy wait to slow down the flicker.
        CLRF    cntr1       ; Reset the counters.
        MOVLW   iter2
        MOVWF   cntr2
; Inner loop.
inner_loop
        DECFSZ  cntr1,F
        GOTO    inner_loop
; Outer loop.
        DECFSZ  cntr2,F
        GOTO    inner_loop
; End of the busy-wait loop: go back to the main loop.
        GOTO    main_loop

; 32-bit Galois LFSR for generating a random sequence.
lfsr
        BCF     STATUS,C
        RRF     lfsr1,F
        RRF     lfsr2,F
        RRF     lfsr3,F
        RRF     lfsr4,F
        BTFSS   STATUS,C
        RETURN  
        MOVLW   tap1
        XORWF   lfsr1,F
        MOVLW   tap2
        XORWF   lfsr2,F
        MOVLW   tap3
        XORWF   lfsr3,F
        MOVLW   tap4
        XORWF   lfsr4,F
        RETURN
        END
